@@ -137,6 +137,8 @@ struct _AdwBottomSheet
   gboolean can_open;
   gboolean can_close;
 
+  gboolean has_been_open;
+
   AdwSwipeTracker *swipe_tracker;
   gboolean swipe_detected;
   gboolean swipe_active;
@@ -598,11 +600,11 @@ adw_bottom_sheet_dispose (GObject *object)
   g_clear_weak_pointer (&self->last_content_focus);
   g_clear_weak_pointer (&self->last_sheet_focus);
 
+  g_clear_object (&self->swipe_tracker);
   g_clear_pointer (&self->content_bin, gtk_widget_unparent);
   g_clear_pointer (&self->dimming, gtk_widget_unparent);
   g_clear_pointer (&self->sheet_bin, gtk_widget_unparent);
   g_clear_object (&self->open_animation);
-  g_clear_object (&self->swipe_tracker);
   self->content = NULL;
   self->sheet = NULL;
   self->sheet_stack = NULL;
@@ -1550,14 +1552,24 @@ adw_bottom_sheet_set_open (AdwBottomSheet *self,
 
   open = !!open;
 
-  if (self->open == open)
+  if (self->open == open) {
+    if (!self->has_been_open && !open) {
+      if (self->closing_callback)
+        self->closing_callback (self, self->user_data);
+
+      if (self->closed_callback)
+        self->closed_callback (self, self->user_data);
+    }
+
     return;
+  }
 
   self->open = open;
 
   if (open) {
     gtk_widget_set_child_visible (self->dimming, self->modal);
     gtk_widget_set_child_visible (self->sheet_bin, TRUE);
+    self->has_been_open = true;
   }
 
   gtk_widget_set_can_target (self->dimming, open);
