@@ -312,11 +312,14 @@ set_focus (AdwDialog *self,
                                           focus_widget_notify_visible_cb, self);
     g_signal_handlers_disconnect_by_func (priv->focus_widget,
                                           focus_widget_notify_parent_cb, self);
+    g_clear_weak_pointer (&priv->focus_widget);
   }
 
   priv->focus_widget = focus;
 
   if (priv->focus_widget) {
+    g_object_add_weak_pointer (G_OBJECT (priv->focus_widget), (gpointer *) &priv->focus_widget);
+
     g_signal_connect_swapped (priv->focus_widget, "hide",
                               G_CALLBACK (unset_focus_widget), self);
     g_signal_connect_swapped (priv->focus_widget, "notify::visible",
@@ -867,9 +870,11 @@ adw_dialog_dispose (GObject *object)
                                           focus_widget_notify_visible_cb, self);
     g_signal_handlers_disconnect_by_func (priv->focus_widget,
                                           focus_widget_notify_parent_cb, self);
-    priv->focus_widget = NULL;
+    g_clear_weak_pointer (&priv->focus_widget);
   }
 
+  g_clear_weak_pointer (&priv->last_focus);
+
   if (priv->default_widget) {
     g_signal_handlers_disconnect_by_func (priv->default_widget,
                                           unset_default_widget, self);
@@ -1894,8 +1899,15 @@ adw_dialog_close (AdwDialog *self)
   }
 
   if (priv->window) {
-    gtk_window_close (GTK_WINDOW (priv->window));
+    GtkWidget *window = priv->window;
     priv->window = NULL;
+
+    if (priv->closing_callback)
+      priv->closing_callback (self, priv->user_data);
+
+    g_signal_emit (self, signals[SIGNAL_CLOSED], 0);
+
+    gtk_window_close (GTK_WINDOW (window));
   } else {
     adw_dialog_force_close (self);
   }
